@@ -21,6 +21,7 @@ import sys
 from collections import OrderedDict
 from typing import Union
 
+from huggingface_hub.utils import is_jinja_available  # noqa: F401
 from packaging import version
 from packaging.version import Version, parse
 
@@ -146,14 +147,6 @@ except importlib_metadata.PackageNotFoundError:
     _unidecode_available = False
 
 
-_modelcards_available = importlib.util.find_spec("modelcards") is not None
-try:
-    _modelcards_version = importlib_metadata.version("modelcards")
-    logger.debug(f"Successfully imported modelcards version {_modelcards_version}")
-except importlib_metadata.PackageNotFoundError:
-    _modelcards_available = False
-
-
 _onnxruntime_version = "N/A"
 _onnx_available = importlib.util.find_spec("onnxruntime") is not None
 if _onnx_available:
@@ -217,6 +210,20 @@ try:
 except importlib_metadata.PackageNotFoundError:
     _k_diffusion_available = False
 
+_wandb_available = importlib.util.find_spec("wandb") is not None
+try:
+    _wandb_version = importlib_metadata.version("wandb")
+    logger.debug(f"Successfully imported wandb version {_wandb_version }")
+except importlib_metadata.PackageNotFoundError:
+    _wandb_available = False
+
+_omegaconf_available = importlib.util.find_spec("omegaconf") is not None
+try:
+    _omegaconf_version = importlib_metadata.version("omegaconf")
+    logger.debug(f"Successfully imported omegaconf version {_omegaconf_version}")
+except importlib_metadata.PackageNotFoundError:
+    _omegaconf_available = False
+
 
 def is_torch_available():
     return _torch_available
@@ -246,10 +253,6 @@ def is_unidecode_available():
     return _unidecode_available
 
 
-def is_modelcards_available():
-    return _modelcards_available
-
-
 def is_onnx_available():
     return _onnx_available
 
@@ -274,6 +277,14 @@ def is_k_diffusion_available():
     return _k_diffusion_available
 
 
+def is_wandb_available():
+    return _wandb_available
+
+
+def is_omegaconf_available():
+    return _omegaconf_available
+
+
 # docstyle-ignore
 FLAX_IMPORT_ERROR = """
 {0} requires the FLAX library but it was not found in your environment. Checkout the instructions on the
@@ -328,6 +339,17 @@ K_DIFFUSION_IMPORT_ERROR = """
 install k-diffusion`
 """
 
+# docstyle-ignore
+WANDB_IMPORT_ERROR = """
+{0} requires the wandb library but it was not found in your environment. You can install it with pip: `pip
+install wandb`
+"""
+
+# docstyle-ignore
+OMEGACONF_IMPORT_ERROR = """
+{0} requires the omegaconf library but it was not found in your environment. You can install it with pip: `pip
+install omegaconf`
+"""
 
 BACKENDS_MAPPING = OrderedDict(
     [
@@ -340,6 +362,8 @@ BACKENDS_MAPPING = OrderedDict(
         ("unidecode", (is_unidecode_available, UNIDECODE_IMPORT_ERROR)),
         ("librosa", (is_librosa_available, LIBROSA_IMPORT_ERROR)),
         ("k_diffusion", (is_k_diffusion_available, K_DIFFUSION_IMPORT_ERROR)),
+        ("wandb", (is_wandb_available, WANDB_IMPORT_ERROR)),
+        ("omageconf", (is_omegaconf_available, OMEGACONF_IMPORT_ERROR)),
     ]
 )
 
@@ -368,10 +392,10 @@ def requires_backends(obj, backends):
 
     if name in [
         "StableDiffusionDepth2ImgPipeline",
-    ] and is_transformers_version("<", "4.26.0.dev0"):
+    ] and is_transformers_version("<", "4.26.0"):
         raise ImportError(
-            f"You need to install `transformers` from 'main' in order to use {name}: \n```\n pip install"
-            " git+https://github.com/huggingface/transformers \n```"
+            f"You need to install `transformers>=4.26` in order to use {name}: \n```\n pip install"
+            " --upgrade transformers \n```"
         )
 
 
@@ -427,12 +451,26 @@ def is_transformers_version(operation: str, version: str):
         operation (`str`):
             A string representation of an operator, such as `">"` or `"<="`
         version (`str`):
-            A string version of PyTorch
+            A version string
     """
     if not _transformers_available:
         return False
     return compare_versions(parse(_transformers_version), operation, version)
 
 
+def is_k_diffusion_version(operation: str, version: str):
+    """
+    Args:
+    Compares the current k-diffusion version to a given reference with an operation.
+        operation (`str`):
+            A string representation of an operator, such as `">"` or `"<="`
+        version (`str`):
+            A version string
+    """
+    if not _k_diffusion_available:
+        return False
+    return compare_versions(parse(_k_diffusion_version), operation, version)
+
+
 class OptionalDependencyNotAvailable(BaseException):
     """An error indicating that an optional dependency of Diffusers was not found in the environment."""
